#!/usr/bin/env python3

#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

"""
Echoes the input, for test purposes
"""

from pathlib import Path

ID =      "echo"
VERSION = "1.0"
INPUT = "anything"
OUTPUT =  ["echo"]

class EchoState():
  def __init__(self):
    self.state = 0
    print(f"initializing echo state: {self.state}...")

  def __str__(self):
    self.inc()
    return f"echo state: {self.state}"

  def inc(self):
    self.state += 1

  def finalize(self):
    self.inc()
    print(f"finalizing echo state: {self.state}...")

def initialize():
  return EchoState()

def finalize(state):
  state.finalize()

def compute(unit, state=None, **kwargs):
  if state:
    print(str(state))
  return [unit], None

def compute_id(filename):
  return Path(filename).stem



#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#
from sqlalchemy import select
from sqlalchemy.schema import MetaData
from pathlib import Path
import yaml
import getpass
import socket
import uuid
import os

ECHO=False
SELFPATH = Path(os.path.abspath(os.path.dirname(__file__)))
TESTDATA = SELFPATH / "testdata"
BIN = SELFPATH / ".." / "bin"
PREFIX = "prenacs_"
PFXAVT = f"{PREFIX}attribute_value_t"

def check_attributes(definitions, connection):
  meta = MetaData()
  meta.reflect(bind=connection)
  adef_t = meta.tables[f"{PREFIX}attribute_definition"]
  rows = connection.execute(select(adef_t.c.name, adef_t.c.datatype,
                                   adef_t.c.computation_group)).all()
  assert(set(r.name for r in rows) == set(definitions.keys()))
  for r in rows:
    assert(r.datatype == definitions[r.name]["datatype"])
    assert(r.computation_group == definitions[r.name]["computation_group"])
  assert(f"{PFXAVT}4" not in meta.tables)
  for n in [0, 1, 2, 3]:
    assert(f"{PFXAVT}{n}" in meta.tables)
  print(set(meta.tables[f"{PFXAVT}0"].c.keys()))
  assert(set(meta.tables[f"{PFXAVT}0"].c.keys()) ==
         {"entity_id", "g1a_v", "g1a_c", "g1b_v", "g1b_c",
                       "g1c_v", "g1c_c", "g1_g"})
  assert(set(meta.tables[f"{PFXAVT}1"].c.keys()) ==
         {"entity_id", "g1d_v", "g1d_c", "g1_g",
                       "g2a_v", "g2a_c", "g2b_v", "g2b_c", "g2_g"})
  assert(set(meta.tables[f"{PFXAVT}2"].c.keys()) ==
         {"entity_id", "g3a_v0", "g3a_v1", "g3a_c", "g3_g"})
  assert(set(meta.tables[f"{PFXAVT}3"].c.keys()) ==
         {"entity_id", "g3b_v0", "g3b_v1", "g3b_v2", "g3b_v3",
                       "g3b_c", "g3_g"})

def check_no_attributes(connection):
  meta = MetaData()
  meta.reflect(bind=connection)
  adef_t = meta.tables[f"{PREFIX}attribute_definition"]
  rows = connection.execute(select(adef_t.c.name, adef_t.c.datatype,
                                   adef_t.c.computation_group)).all()
  assert(len(rows) == 0)
  assert(f"{PFXAVT}4" not in meta.tables)
  for n in [0, 1, 2, 3]:
    assert(f"{PFXAVT}{n}" in meta.tables)
    assert(set(meta.tables[f"{PFXAVT}{n}"].c.keys()) == {"entity_id"})

def computation_id(n):
  return uuid.UUID(f'00000000-0000-0000-0000-00000000000{n}').bytes

def get_attribute_value_rows(connection):
  meta = MetaData()
  meta.reflect(bind=connection)
  tnums = [0,1,2,3]
  r = []
  for n in tnums:
    t = meta.tables[f"{PFXAVT}{n}"]
    rows = connection.execute(select(t)).all()
    r.append({row.entity_id: row for row in rows})
  return r

def check_g1_t0(row, run):
  if run != 2:
    assert(row.g1a_v == 1)
    assert(row.g1b_v == 2)
    assert(row.g1c_v == 3)
    # computation ID in _g and not _c, as whole grp computed
    assert(row.g1_g == computation_id(1))
    for field in ["g1a_c", "g1b_c", "g1c_c"]:
      assert(getattr(row, field) == None)
  else:
    # computation IDs stored in _c for re-computed values
    # as not entire group recomputed
    assert(row.g1a_v == 5)
    assert(row.g1a_c == computation_id(2))
    assert(row.g1b_v == 2)
    assert(row.g1b_c == None)
    assert(row.g1_g == computation_id(1))
    assert(row.g1c_v == 6)
    assert(row.g1c_c == computation_id(2))

def check_g1_t1(row, run):
  assert(row.g1d_c == None)
  if run != 2:
    assert(row.g1d_v == 4)
    assert(row.g1_g == computation_id(1))
  else:
    assert(row.g1d_v == 7)
    assert(row.g1_g == computation_id(2))

def check_g2_t1(row, run):
  if run < 3:
    for field in ["g2a_v", "g2a_c", "g2b_v", "g2b_c", "g2_g"]:
      assert(getattr(row, field) == None)
  else:
    assert(row.g2a_v == 8)
    assert(row.g2a_c == None)
    assert(row.g2_g == computation_id(3))
    if run == 3:
      assert(row.g2b_v == 9)
      assert(row.g2b_c == None)
    else:
      assert(row.g2b_v == 10)
      assert(row.g2b_c == computation_id(4))

def check_g3_t2(row, run):
  assert(row.g3a_v0 == 10)
  assert(row.g3a_v1 == 2.3)
  assert(row.g3a_c == None)
  if run <= 6:
    assert(row.g3_g == computation_id(5))
  else:
    assert(row.g3_g == computation_id(7))

def check_g3_t3(row, run):
  assert(row.g3b_v0 == 1.0)
  assert(row.g3b_v1 == 11)
  assert(row.g3b_v2 == 12)
  assert(row.g3b_v3 == 13)
  assert(row.g3b_c == None)
  if run == 5:
    assert(row.g3_g == computation_id(5))
  else:
    assert(row.g3_g == computation_id(6))

def check_values_after_run(n, connection):
  r = get_attribute_value_rows(connection)
  for accession in ['A1','A2','A3','A4']:
    check_g1_t0(r[0][accession], n)
    check_g1_t1(r[1][accession], n)
    check_g2_t1(r[1][accession], n)
  if n <= 4:
    assert(r[2] == {})
    assert(r[3] == {})
  else:
    for accession in ['A1','A2','A3','A4']:
      check_g3_t2(r[2][accession], n)
      check_g3_t3(r[3][accession], n)

def check_results(outfilename, expectedfilename, basename=False):
  results = {}
  expected = {}
  with open(expectedfilename) as f:
    for line in f:
      elems = line.rstrip().split("\t")
      key = elems.pop(0)
      expected[key] = elems
  with open(outfilename) as f:
    for line in f:
      elems = line.rstrip().split("\t")
      entity_id = elems.pop(0)
      if basename:
        entity_id = os.path.basename(entity_id)
      results[entity_id] = elems
  assert(results == expected)

def check_file_content(filename, expected):
  with open(filename) as f:
    assert(f.read() == expected)

def check_empty_file(filename):
  check_file_content(filename, "")

def check_report(reportfilename,
                 plugin_id, plugin_version, n_units, comp_status, reason=None,
                 user_id=getpass.getuser(), system_id=socket.gethostname(),
                 params={}):
  with open(reportfilename) as f:
    report = yaml.safe_load(f)
  assert(report["plugin_id"] == plugin_id)
  assert(report["plugin_version"] == plugin_version)
  assert(report["n_units"] == n_units)
  assert(report["comp_status"] == comp_status)
  report_params = yaml.safe_load(report["parameters"])
  assert(report_params == params)
  assert(report["user_id"] == user_id)
  assert(report["system_id"] == system_id)
  assert(report["reason"] == reason)




#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, Text
from sqlalchemy_repr import PrettyRepresentableBase

Base = declarative_base(cls=PrettyRepresentableBase)

utf8_cs_args = {'mysql_charset': 'utf8', 'mysql_collate': 'utf8_bin'}

class PluginDescription(Base):
  """
  Describes a plugin for assembly attributes computation.
  """
  __tablename__ = "prenacs_plugin_description"
  id = Column(String(256), primary_key=True)
  version = Column(String(64), primary_key=True)
  input = Column(String(512), nullable=False)
  output = Column(String(512), nullable=False)
  method = Column(Text(4096))
  implementation = Column((Text(4096)))
  parameters = Column((Text(4096)))
  req_software = Column(Text(4096))
  req_hardware = Column(Text(4096))
  advice = Column(Text(4096))
  __table_args__ = utf8_cs_args




#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

import pytest
import os
from sqlalchemy import create_engine
from sqlalchemy.engine.url import URL
from attrtables import AttributeValueTables
import prenacs.database
from prenacs import AttributeDefinition
import yaml

VERBOSE_CONNECTION = True
DEBUG_MODE = False

def connection_data_from_config():
  # if config.yaml does not exist, raise an error
  config_file_path = os.path.join(os.path.dirname(__file__), 'config.yaml')
  if not os.path.exists(config_file_path):
    raise FileNotFoundError(\
        'Connection configuration file config.yaml not found')
  with open(config_file_path) as f:
    config = yaml.safe_load(f)
  return config

@pytest.fixture(scope="session")
def connection_string():
  config = connection_data_from_config()
  args = {k: v for k, v in config.items() if k in ['drivername',
                                           'host', 'port', 'database',
                                           'username', 'password']}
  if 'socket' in config:
    args['query'] = {'unix_socket': config['socket']}
  return URL.create(**args)

@pytest.fixture()
def connection_args():
  config = connection_data_from_config()
  return [config['username'], config['password'],
          config['database'], config['socket']]

def drop_all(connection):
  avt = AttributeValueTables(connection,
                             attrdef_class=AttributeDefinition,
                             tablename_prefix="pvt_attribute_value_t")
  prenacs.database.drop(connection, avt)

@pytest.fixture()
def connection_creator(connection_string):
  def get_connection():
    engine = create_engine(connection_string, echo=VERBOSE_CONNECTION,
                           future=True)
    connection = engine.connect()
    return connection
  return get_connection

@pytest.fixture(scope="session")
def connection(connection_string):
  engine = create_engine(connection_string, echo=VERBOSE_CONNECTION,
                         future=True)
  with engine.connect() as conn:
    drop_all(conn)
    conn.commit()
    prenacs.database.create(conn)
    conn.commit()
    yield conn
    if not DEBUG_MODE:
      drop_all(conn)
    conn.commit()



#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

"""
DB Schema for tables storing attribute values.
"""
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, String, Text
from sqlalchemy_repr import PrettyRepresentableBase

Base = declarative_base(cls=PrettyRepresentableBase)

utf8_cs_args = {'mysql_charset': 'utf8', 'mysql_collate': 'utf8_bin'}

class AttributeDefinition(Base):
  """
  Describes an assembly attribute.

  If no definition is provided, the definition is given by the
  ontology link; otherwise the ontology link is a term which is
  related to the definition, in a way described in the definition.
  """
  __tablename__ = "prenacs_attribute_definition"
  name = Column(String(62), primary_key=True)
  datatype = Column(String(256), nullable=False)
  definition = Column(Text(4096))
  ontology_xref = Column(String(64))
  related_ontology_terms = Column(Text(4096))
  unit = Column(String(64))
  remark = Column(Text(4096))
  computation_group = Column(String(62), index=True)
  __table_args__ = utf8_cs_args




#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, DateTime, Text, Enum
from sqlalchemy_repr import PrettyRepresentableBase
from sqlalchemy.dialects.mysql import BINARY

Base = declarative_base(cls=PrettyRepresentableBase)

utf8_cs_args = {'mysql_charset': 'utf8', 'mysql_collate': 'utf8_bin'}

class ComputationReport(Base):
  """
  Report of a computation.

  Tables storing the results of computations can refer to this table for the
  sake of data traceability and computation reproducibility.
  """
  __tablename__ = 'prenacs_computation_report'
  uuid = Column(BINARY(16), primary_key=True)
  plugin_id = Column(String(256), nullable=False)
  plugin_version = Column(String(64), nullable=False)
  parameters = Column(Text(4096))
  unit = Column(Enum("assembly"), nullable=False, default="assembly")
  n_units = Column(Integer, default=1)
  reason = Column(Enum("new_entities", "new_attributes", "recompute"))
  comp_status = Column(Enum("running", "completed", "partial", "aborted"),
                  nullable=False, default="running")
  system_id = Column(String(64))
  user_id = Column(String(64))
  time_start = Column(DateTime)
  time_end = Column(DateTime)
  used_resources = Column(Text(4096))
  remarks = Column(Text(4096))
  __table_args__ = utf8_cs_args




#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

import pytest
import yaml
from helper import TESTDATA, check_attributes, ECHO, \
                   BIN, check_values_after_run, check_no_attributes, \
                   check_results, check_file_content, check_empty_file, \
                   check_report
import os
import tempfile
from contextlib import contextmanager

@pytest.mark.script_launch_mode('subprocess')
def test_prenacs_cli(connection_creator, connection_args, script_runner):
  ret = script_runner.run(str(BIN/"prenacs-setup-database"), *connection_args)
  assert(ret.returncode == 0)
  attrs = connection_args + [TESTDATA/"fake_attrs.yaml", "--testmode"]
  ret = script_runner.run(str(BIN/"prenacs-manage-attributes"), *attrs)
  assert(ret.returncode == 0)
  with open(TESTDATA/"fake_attrs.yaml") as f:
    definitions = yaml.safe_load(f)
  connection = connection_creator()
  check_attributes(definitions, connection)
  connection.close()
  for testn in [1, 2, 1, 3, 4, 5, 6, 7]:
    attrs = connection_args + [TESTDATA/f"fake_results{testn}.tsv",
                               TESTDATA/f"fake_report{testn}.yaml",
                               TESTDATA/f"fake_plugin{testn}.py"]
    script_runner.run(str(BIN/"prenacs-load-results"), *attrs)
    connection = connection_creator()
    check_values_after_run(testn, connection)
    connection.close()
  attrs = connection_args + [TESTDATA/"empty_dict.yaml", "--drop", "--testmode"]
  ret = script_runner.run(str(BIN/"prenacs-manage-attributes"), *attrs)
  assert(ret.returncode == 0)
  connection = connection_creator()
  check_no_attributes(connection)
  connection.close()
  ret = script_runner.run(str(BIN/"prenacs-destroy-database"), *connection_args)

@contextmanager
def auto_args(parallel, verbose, params = None):
  reportfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  reportfile.close()
  outfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  outfile.close()
  logfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  logfile.close()
  args = ["--out", outfile.name, "--report", reportfile.name,
         "--log", logfile.name]
  if not parallel:
    args.append("--serial")
  if verbose:
    args.append("--verbose")
  if params:
    paramsfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
    yaml.dump(params, paramsfile)
    paramsfile.close()
    args += ["--params", paramsfile.name]
    yield (args, outfile.name, reportfile.name, logfile.name, paramsfile.name)
    os.unlink(paramsfile.name)
  else:
    yield (args, outfile.name, reportfile.name, logfile.name, None)
  os.unlink(outfile.name)
  os.unlink(logfile.name)
  os.unlink(reportfile.name)

@pytest.mark.script_launch_mode('subprocess')
def test_prenacs_cli_batch_computing_files(script_runner):
  for parallel in [False, True]:
    with auto_args(parallel, ECHO) as \
        (args, outfilename, reportfilename, logfilename, paramsfilename):
      ret = script_runner.run(str(BIN/"prenacs-batch-compute"),
                              str(TESTDATA/"wc_from_filename_plugin.sh"),
                              "files", str(TESTDATA/"*.data"), *args)
      assert(ret.returncode == 0)
      check_report(reportfilename, "wc", "1.0", 9, "completed")
      check_results(outfilename, str(TESTDATA/"wc_expected_wfilename.tsv"),
                    basename=True)
      check_empty_file(logfilename)

@pytest.mark.script_launch_mode('subprocess')
def test_prenacs_cli_batch_computing_ids(script_runner):
  for parallel in [False, True]:
    params = {"testdatadir": str(TESTDATA)}
    with auto_args(parallel, ECHO, params) as \
        (args, outfilename, reportfilename, logfilename, paramsfilename):
      ret = script_runner.run(str(BIN/"prenacs-batch-compute"),
                              str(TESTDATA/"wc_from_id_plugin.sh"),
                              "ids", str(TESTDATA/"ids.tsv"),
                              "--system", "testsystem", *args)
      assert(ret.returncode == 0)
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   system_id="testsystem")
      check_results(outfilename, str(TESTDATA/"wc_expected.tsv"))
      check_empty_file(logfilename)

@pytest.mark.script_launch_mode('subprocess')
def test_prenacs_cli_batch_computing_ids_with_error_in_one_unit(script_runner):
  for parallel in [False, True]:
    params = {"testdatadir": str(TESTDATA)}
    with auto_args(parallel, ECHO, params) as \
        (args, outfilename, reportfilename, logfilename, paramsfilename):
      ret = script_runner.run(str(BIN/"prenacs-batch-compute"),
                              str(TESTDATA/"wc_from_id_plugin.sh"),
                              "ids", str(TESTDATA/"ids.tsv"), "2",
                              "--user", "testuser", *args)
      assert(ret.returncode == 0)
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   user_id="testuser")
      check_results(outfilename, str(TESTDATA/"wc_expected_wo_9.tsv"))
      check_file_content(logfilename,
          f"0\t{TESTDATA}/input0.data does not exist\n")

@pytest.mark.script_launch_mode('subprocess')
def test_prenacs_cli_batch_computing_ids_with_proc(script_runner):
  for parallel in [False, True]:
    params = {"testdatadir": str(TESTDATA)}
    with auto_args(parallel, ECHO, params) as \
        (args, outfilename, reportfilename, logfilename, paramsfilename):
      ret = script_runner.run(str(BIN/"prenacs-batch-compute"),
                              str(TESTDATA/"wc_from_id_plugin.sh"),
                              "ids", str(TESTDATA/"ids.tsv"), "2",
                              "--idsproc", str(TESTDATA/"one_adder_idsproc.sh"),
                              "--reason", "new_attributes", *args)
      assert(ret.returncode == 0)
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   reason="new_attributes")
      check_results(outfilename, str(TESTDATA/"wc_expected.tsv"))
      check_empty_file(logfilename)



#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

import yaml
from attrtables import AttributeValueTables
from prenacs import AttributeDefinition, AttributeDefinitionsManager,\
                      ResultsLoader, BatchComputation
from helper import PFXAVT, ECHO, TESTDATA, check_attributes, \
                   check_values_after_run, check_no_attributes, \
                   check_results, check_file_content, check_empty_file, \
                   check_report
import tempfile
import os
from contextlib import contextmanager

def test_prenacs_api_database(connection):
  avt = AttributeValueTables(connection,
                             attrdef_class=AttributeDefinition,
                             tablename_prefix=PFXAVT)
  avt.target_n_columns = 9
  adm = AttributeDefinitionsManager(avt)
  with open(TESTDATA/"fake_attrs.yaml") as f:
    definitions = yaml.safe_load(f)
  adm.apply_definitions(definitions)
  connection.commit()
  check_attributes(definitions, connection)
  for testn in [1, 2, 1, 3, 4, 5, 6, 7]:
    results_loader = ResultsLoader(avt, TESTDATA/f"fake_plugin{testn}.py",
                                   verbose=ECHO)
    results_loader.run(TESTDATA/f"fake_results{testn}.tsv",
                       TESTDATA/f"fake_report{testn}.yaml",
                       verbose=ECHO)
    connection.commit()
    check_values_after_run(testn, connection)
  adm.apply_definitions({})
  connection.commit()
  check_no_attributes(connection)
  avt.drop_all()

@contextmanager
def outfiles(bc, **kwargs):
  reportfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  outfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  outfile.close()
  logfile = tempfile.NamedTemporaryFile(mode="w", delete=False)
  logfile.close()
  bc.setup_computation(reportfile=reportfile, **kwargs)
  bc.set_output(outfilename=outfile.name, logfilename=logfile.name)
  yield outfile.name, logfile.name, reportfile.name
  os.unlink(outfile.name)
  os.unlink(logfile.name)
  os.unlink(reportfile.name)

def test_prenacs_api_batch_computing_files():
  for parallel in [True, False]:
    bc = BatchComputation(str(TESTDATA/"wc_from_filename_plugin.sh"))
    bc.input_from_globpattern(str(TESTDATA/"*.data"), verbose=ECHO)
    with outfiles(bc) as (outfilename, logfilename, reportfilename):
      bc.run(verbose=ECHO, parallel=parallel)
      bc.finalize()
      check_report(reportfilename, "wc", "1.0", 9, "completed")
      check_results(outfilename, str(TESTDATA/"wc_expected_wfilename.tsv"),
                    basename=True)
      check_empty_file(logfilename)

def test_prenacs_api_batch_computing_ids():
  for parallel in [True, False]:
    bc = BatchComputation(str(TESTDATA/"wc_from_id_plugin.sh"))
    bc.input_from_idsfile(str(TESTDATA/"ids.tsv"), verbose=ECHO)
    params = {"testdatadir": str(TESTDATA)}
    with outfiles(bc, params=params, system="testsystem") as \
        (outfilename, logfilename, reportfilename):
      bc.run(verbose=ECHO, parallel=parallel)
      bc.finalize()
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   system_id="testsystem")
      check_results(outfilename, str(TESTDATA/"wc_expected.tsv"))
      check_empty_file(logfilename)

def test_prenacs_api_batch_computing_ids_with_error_in_one_unit():
  for parallel in [True, False]:
    bc = BatchComputation(str(TESTDATA/"wc_from_id_plugin.sh"))
    bc.input_from_idsfile(str(TESTDATA/"ids.tsv"), 2, verbose=ECHO)
    params = {"testdatadir": str(TESTDATA)}
    with outfiles(bc, params=params, user="testuser") as \
        (outfilename, logfilename, reportfilename):
      bc.run(verbose=ECHO, parallel=parallel)
      bc.finalize()
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   user_id="testuser")
      check_results(outfilename, str(TESTDATA/"wc_expected_wo_9.tsv"))
      check_file_content(logfilename,
          f"0\t{TESTDATA}/input0.data does not exist\n")

def test_prenacs_api_batch_computing_ids_with_proc():
  for parallel in [True, False]:
    bc = BatchComputation(str(TESTDATA/"wc_from_id_plugin.sh"))
    bc.input_from_idsfile(str(TESTDATA/"ids.tsv"), 2, verbose=ECHO,
                          idsproc_module=str(TESTDATA/"one_adder_idsproc.sh"))
    params = {"testdatadir": str(TESTDATA)}
    with outfiles(bc, params=params, reason="new_attributes") as \
        (outfilename, logfilename, reportfilename):
      bc.run(verbose=ECHO, parallel=parallel)
      bc.finalize()
      check_report(reportfilename, "wc", "1.0", 9, "completed", params=params,
                   reason="new_attributes")
      check_results(outfilename, str(TESTDATA/"wc_expected.tsv"))
      check_empty_file(logfilename)



#!/usr/bin/env python3

#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

ID="wc"
VERSION="1.0"
INPUT="filename"
OUTPUT=["lines", "words", "bytes"]
IMPLEMENTATION="based on Posix tool wc"
REQ_SOFTWARE="wc, pip library sh"

from sh import wc

def compute(filename, **kwargs):
  wc_out=wc(filename).split()
  return "\t".join(wc_out[0:3]), ""



#!/usr/bin/env python3

#
# (c) 2021-2022 Giorgio Gonnella, University of Goettingen, Germany
#

"""
Does nothing, for test purposes
"""

ID =      "fake5"
VERSION = "0.1.0"
INPUT = "test"
OUTPUT =  ["g3a", "g3b"]

def compute(filename, **kwargs):
  return [10, 2.3, 1.0, 11, 12, 13], ["test\tthis is a test by fake5"]

def fake_data():
  for acc in ["A1","A2","A3","A4"]:
    print("\t".join([acc]+[str(e) for e in compute(acc)[0]]))

if __name__ == "__main__":
  fake_data()
